<template>
	<div class='widget-wrapper' :style='{...widgetWrapperStyle, pointerEvents: lockClick ? "all" : "none"}'>
		<div class="widget" :style='{...widgetStyle, ...transitionStyle}'>
			<div class="title" v-html="title"/>

			<div class="scale" @mousedown.stop="mouseDown($event)">
				<div
					class="average-response"
					:class='{disabled: value !== null}'
					:style="{left: selectedValue + '%'}"
				>
					<div
						class="smile"
						:style='{borderColor: "hsl(" + hue + ", 100%, 50%)"}'
					></div>
				</div>

				<div class="bar" :style='barStyle'>
					<div class="line" :style="{width: (100 - this.selectedValue) + '%'}"></div>
				</div>
			</div>

			<div class='subtitle-wrapper' :class='{hidden: value === null}'>
				<div class="subtitle">Средняя оценка — <span class='value'>{{ Math.round(value) }}%</span></div>
			</div>

		</div>
	</div>
</template>

<script>
import {widgetMixin, positionMixin} from '../helpers/widgets.mjs'

export default {
	mixins: [widgetMixin, positionMixin],

	props: {
		title: {
			required: true,
			type: String,
		},
	},

	data() {
		return {
			selectedValue: 50,
			loadedValue: false,
			value: null,
			startHue: 0,
			amplitude: 100,
			lockClick: false,
		}
	},

	beforeUnmount() {
		clearInterval(this.updateInterval)
	},

	methods: {
		vote() {
			this.act('react', {value: parseInt(this.selectedValue)}).then((data) => {
				const values = data.data

				this.setData(values)
				this.loadedValue = true

				this.updateInterval = setInterval(this.getData, 3000)
			})
		},

		mouseDown() {
			if (this.value !== null) return

			const block = document.querySelector('.scale')
			const w = block.clientWidth
			const left = block.getBoundingClientRect().left

			// Вызывается при движении мыши
			const move = (e) => {
				this.disableWatch = true
				this.lockClick = true
				// Получаем разницу между текущим положением мыши и начальным
				const dx = e.pageX - left

				this.selectedValue = dx / w * 100
				if (this.selectedValue < 0) this.selectedValue = 0
				if (this.selectedValue > 100) this.selectedValue = 100
			}

			document.addEventListener('mousemove', move)

			const up = (e) => {
				e.preventDefault()
				e.stopPropagation()

				this.lockClick = false

				// При отпускании мыши убираем обработчики на движение и отпускание
				document.removeEventListener('mousemove', move)
				document.removeEventListener('mouseup', up)

				this.selectedValue = parseInt(this.selectedValue)
				this.vote()
			}

			// Устанавливаем обработчик up на отпускание мыши
			document.addEventListener('mouseup', up)
		},
	},

	computed: {
		hue() {
			return this.startHue + (this.amplitude * this.selectedValue / 100)
		},

		barStyle() {
			const gradient = `
linear-gradient(to right, hsl(${this.startHue}, 100%, 50%) 0%, hsl(${this.startHue + this.amplitude / 2}, 100%, 50%) 50%,
hsl(${this.startHue + this.amplitude}, 100%, 50%) 100%)
`

			return {
				background: gradient,
			}
		},
	},
}
</script>

<style scoped>

.widget {
	border-radius: .7em;
	color: white;
	background: rgba(0, 0, 0, 0.15);
	backdrop-filter: blur(20px);

	font-family: var(--primary-font);
	font-variation-settings: 'wght' 300;

	width: 14em;

	padding: 1em 1.5em;

	display: flex;
	flex-direction: column;
	gap: 1.5em;
	font-size: .9em;

	pointer-events: all;
}

.title {
	text-align: center;
}

.scale {
	position: relative;
}

.average-response {
	position: absolute;
	top: 0;
	left: 0;
	transform: translate(-50%, calc(-50% + .3em));
	z-index: 1;

	cursor: grab;
}

.disabled {
	cursor: default;
}

.smile {
	width: 1.5em;
	height: 1.5em;
	border-radius: 50%;
	background: white;
	border: .2em solid currentColor;
}

.bar {
	height: .6em;

	border-radius: 1.27em;
	overflow: hidden;

	display: flex;
	justify-content: flex-end;
}

.line {
	height: 100%;
	background: white;
}

.subtitle-wrapper {
	max-height: 3em;
	overflow: hidden;
	transition: max-height .2s ease, margin-top .2s ease;
}

.subtitle-wrapper.hidden {
	max-height: 0;
	margin-top: -1em;
}

.subtitle {
	text-align: center;
}

.subtitle .value {
	font-variation-settings: 'wght' 600;
}

</style>
